@@ -20,11 +20,12 @@
 #include <bonobo/bonobo-ui-engine.h>
 #include <bonobo/bonobo-ui-sync.h>
 #include <bonobo/bonobo-ui-sync-menu.h>
+#include <bonobo/bonobo-ui-private.h>
 #include <bonobo/bonobo-ui-preferences.h>
 
 #undef WIDGET_SYNC_DEBUG
 
-static BonoboUISyncClass *parent_class = NULL;
+static GObjectClass *parent_class = NULL;
 
 #define PARENT_TYPE bonobo_ui_sync_get_type ()
 
@@ -109,22 +110,22 @@ node_is_popup (BonoboUINode *node)
 static void
 add_tearoff (BonoboUINode *node, GtkMenu *menu, gboolean popup_init)
 {
-	GtkWidget    *tearoff;
-	char         *txt;
-	gboolean      has_tearoff;
+	GtkWidget *tearoff;
+	gboolean   has_tearoff;
 
 	has_tearoff = bonobo_ui_preferences_get_menus_have_tearoff ();
 
 	if (node) {
-		txt = bonobo_ui_node_get_attr (node, "tearoff");
+		const char *txt;
+
+		txt = bonobo_ui_node_peek_attr (node, "tearoff");
 
 		if (txt)
 			has_tearoff = atoi (txt);
+
 		else if (node_is_popup (node))
 			has_tearoff = FALSE;
 
-		bonobo_ui_node_free_string (txt);
-
 	} else if (popup_init)
 		has_tearoff = FALSE;
 
@@ -257,24 +258,18 @@ get_item_widget (GtkWidget *widget)
 	return NULL;
 }
 
-static GtkWidget *
-cmd_get_menu_pixmap (BonoboUINode     *node,
-		     BonoboUINode     *cmd_node)
+static GdkPixbuf *
+cmd_get_menu_pixbuf (GtkWidget    *widget,
+		     BonoboUINode *node,
+		     BonoboUINode *cmd_node)
 {
-	GtkWidget *pixmap;
-	char      *type;
+	if (bonobo_ui_node_peek_attr (node, "pixtype"))
+		return bonobo_ui_util_xml_get_pixbuf (
+			widget, node, GTK_ICON_SIZE_MENU);
 
-	if ((type = bonobo_ui_node_get_attr (node, "pixtype"))) {
-		pixmap = bonobo_ui_util_xml_get_icon_widget (node, GTK_ICON_SIZE_MENU);
-		bonobo_ui_node_free_string (type);
-		return pixmap;
-	}
-
-	if ((type = bonobo_ui_node_get_attr (cmd_node, "pixtype"))) {
-		pixmap = bonobo_ui_util_xml_get_icon_widget (cmd_node, GTK_ICON_SIZE_MENU);
-		bonobo_ui_node_free_string (type);
-		return pixmap;
-	}
+	if (bonobo_ui_node_peek_attr (cmd_node, "pixtype"))
+		return bonobo_ui_util_xml_get_pixbuf (
+			widget, cmd_node, GTK_ICON_SIZE_MENU);
 
 	return NULL;
 }
@@ -351,7 +346,7 @@ impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 			menu_widget = widget;
 
 		/* Recurse here just once, don't duplicate in the build. */
-		bonobo_ui_engine_update_node (engine, node);
+		bonobo_ui_engine_update_node (engine, sync, node);
 
 	} else if (bonobo_ui_node_has_name (node, "menuitem"))
 		menu_widget = widget;
@@ -362,13 +357,26 @@ impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 		bonobo_ui_node_free_string (type);
 	else {
 		if (GTK_IS_IMAGE_MENU_ITEM (menu_widget)) {
-			GtkWidget *pixmap;
+			GdkPixbuf        *pixbuf;
+			GtkImageMenuItem *image_menu_item;
 
-			pixmap = cmd_get_menu_pixmap (node, cmd_node);
+			pixbuf = cmd_get_menu_pixbuf (menu_widget, node, cmd_node);
 
-			if (pixmap) {
-				gtk_widget_show (GTK_WIDGET (pixmap));
-				g_object_set (G_OBJECT (menu_widget), "image", pixmap, NULL);
+			image_menu_item = (GtkImageMenuItem *) menu_widget;
+
+			if (pixbuf) {
+				GtkWidget *image;
+			       
+				image = gtk_image_menu_item_get_image (image_menu_item);
+
+				if (!image) {
+					image = gtk_image_new_from_pixbuf (pixbuf);
+					gtk_widget_show (image);
+					g_object_set (G_OBJECT (menu_widget), "image",
+						      image, NULL);
+				} else
+					bonobo_ui_image_set_pixbuf (
+						GTK_IMAGE (image), pixbuf);
 			}
 		}
 	}
@@ -753,6 +761,29 @@ radio_group_destroy (gpointer	key,
 	return TRUE;
 }
 
+static void
+impl_dispose (GObject *object)
+{
+	BonoboUISyncMenu *sync = (BonoboUISyncMenu *) object;
+
+	if (sync->menu) {
+		g_object_unref (G_OBJECT (sync->menu));
+		sync->menu = NULL;
+	}
+
+	if (sync->menu_dock_item) {
+		g_object_unref (G_OBJECT (sync->menu_dock_item));
+		sync->menu_dock_item = NULL;
+	}
+
+	if (sync->accel_group) {
+		g_object_unref (G_OBJECT (sync->accel_group));
+		sync->accel_group = NULL;
+	}
+
+	parent_class->dispose (object);
+}
+
 static void
 impl_finalize (GObject *object)
 {
@@ -765,7 +796,7 @@ impl_finalize (GObject *object)
 	g_hash_table_destroy (sync->radio_groups);
 	sync->radio_groups = NULL;
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	parent_class->finalize (object);
 }
 
 static gboolean
@@ -853,6 +884,7 @@ class_init (BonoboUISyncClass *sync_class)
 	parent_class = g_type_class_peek_parent (sync_class);
 
 	object_class = G_OBJECT_CLASS (sync_class);
+	object_class->dispose  = impl_dispose;
 	object_class->finalize = impl_finalize;
 
 	sync_class->sync_state = impl_bonobo_ui_sync_menu_state;
@@ -914,9 +946,13 @@ bonobo_ui_sync_menu_new (BonoboUIEngine *engine,
 
 	sync = g_object_new (BONOBO_TYPE_UI_SYNC_MENU, NULL);
 
-	sync->menu           = menu;
-	sync->menu_dock_item = menu_dock_item;
-	sync->accel_group    = group;
+	sync->menu = menu ? g_object_ref (G_OBJECT (menu)) : NULL;
+
+	sync->menu_dock_item = menu_dock_item ?
+		g_object_ref (G_OBJECT (menu_dock_item)) :
+		menu_dock_item;
+
+	sync->accel_group = group ? g_object_ref (G_OBJECT (group)) : NULL;
 
 	return bonobo_ui_sync_construct (
 		BONOBO_UI_SYNC (sync), engine, TRUE, TRUE);
